#  Copyright (C) 2016 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class Clutter(Tarball, Project):
    def __init__(self):
        Project.__init__(
            self,
            "clutter",
            repository="https://gitlab.gnome.org/Archive/clutter",
            version="1.26.4",
            lastversion_even=True,
            archive_url="https://download.gnome.org/sources/clutter/{major}.{minor}/clutter-{version}.tar.xz",
            hash="8b48fac159843f556d0a6be3dbfc6b083fc6d9c58a20a49a6b4919ab4263c4e6",
            dependencies=["atk", "cogl", "json-glib"],
        )

    def build(self):
        self.builder.mod_env("INCLUDE", f"{self.builder.gtk_dir}\\include\\harfbuzz")
        self.exec_msbuild_gen(r"build\win32", "clutter.sln", add_pars="/p:UseEnv=True")

        self.install(r".\COPYING share\doc\clutter")



#  Copyright (C) 2016 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

from gvsbuild.utils.base_builders import Meson
from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import project_add


@project_add
class Dav1d(Tarball, Meson):
    def __init__(self):
        Meson.__init__(
            self,
            "dav1d",
            version="1.5.1",
            archive_url="https://code.videolan.org/videolan/dav1d/-/archive/{version}/dav1d-{version}.tar.gz",
            hash="fa635e2bdb25147b1384007c83e15de44c589582bb3b9a53fc1579cb9d74b695",
            dependencies=[
                "nasm",
                "ninja",
                "meson",
                "pkgconf",
            ],
        )
        self.add_param("-Denable_tests=false")
        self.add_param("-Denable_tools=false")

    def build(self):
        Meson.build(self)



#  Copyright (C) 2016 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

from gvsbuild.utils.base_builders import CmakeProject
from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class Libcbor(Tarball, CmakeProject):
    def __init__(self):
        Project.__init__(
            self,
            "libcbor",
            version="0.12.0",
            archive_url="https://github.com/PJK/libcbor/archive/refs/tags/v{version}.tar.gz",
            archive_filename="libcbor-{version}.tar.gz",
            hash="5368add109db559f546d7ed10f440f39a273b073daa8da4abffc83815069fa7f",
            dependencies=["cmake", "ninja"],
        )

    def build(self):
        # If do_install is True, the build fails
        CmakeProject.build(self, use_ninja=True, do_install=False)
        self.install(r"_gvsbuild-cmake\src\cbor.lib lib")
        self.install(r"_gvsbuild-cmake\src\cbor\*.h include\cbor")
        self.install(r"_gvsbuild-cmake\cbor\*.h include\cbor")
        self.install(r"src\cbor.h include")
        self.install(r"src\cbor\*.h include\cbor")



#  Copyright (C) 2016 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

from gvsbuild.utils.base_builders import Meson
from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import project_add


@project_add
class Glibmm(Tarball, Meson):
    def __init__(self):
        Meson.__init__(
            self,
            "glibmm",
            prj_dir="glibmm",
            version="2.84.0",
            lastversion_even=True,
            repository="https://gitlab.gnome.org/GNOME/glibmm",
            archive_url="https://download.gnome.org/sources/glibmm/{major}.{minor}/glibmm-{version}.tar.xz",
            hash="56ee5f51c8acfc0afdf46959316e4c8554cb50ed2b6bc5ce389d979cbb642509",
            dependencies=[
                "meson",
                "ninja",
                "libsigc++",
                "glib",
            ],
            patches=[
                # Fixed in https://gitlab.gnome.org/GNOME/gtkmm/-/commit/9490a86b42b7a980ca8cfabaa63be0655071c546
                "001-fix-python-not-found.patch",
            ],
        )

    def build(self):
        Meson.build(
            self,
            meson_params="-Dbuild-examples=false -Dbuild-documentation=false",
        )

        self.install(r".\COPYING share\doc\glibmm")



#  Copyright (C) 2016 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

from gvsbuild.utils.base_builders import Meson
from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class Libgxps(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "libgxps",
            version="0.3.2",
            archive_url="https://download.gnome.org/sources/libgxps/{major}.{minor}/libgxps-{version}.tar.xz",
            hash="6d27867256a35ccf9b69253eb2a88a32baca3b97d5f4ef7f82e3667fa435251c",
            dependencies=[
                "meson",
                "ninja",
                "pkgconf",
                "glib",
                "libarchive",
                "cairo",
                "libpng",
                "libjpeg-turbo",
                "libtiff-4",
            ],
            patches=[
                "0001-converter-pdf-add-option-for-stdout-output.patch",
            ],
        )
        if self.opts.enable_gi:
            self.add_dependency("gobject-introspection")
            disable_gi = "false"
        else:
            disable_gi = "true"

        self.add_param(f"-Ddisable-introspection={disable_gi}")
        self.add_param("-Dwith-liblcms2=false")
        self.add_param("-Denable-test=false")

    def build(self):
        Meson.build(self)

        self.install(r".\COPYING share\doc\libgxps")



#  Copyright (C) 2016 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

import os

from gvsbuild.utils.base_expanders import GitRepo
from gvsbuild.utils.base_project import Project, project_add
from gvsbuild.utils.utils import convert_to_msys


@project_add
class X264(GitRepo, Project):
    def __init__(self):
        Project.__init__(
            self,
            "x264",
            repo_url="http://git.videolan.org/git/x264.git",
            fetch_submodules=False,
            dependencies=["nasm", "msys2"],
            tag="31e19f92f00c7003fa115047ce50978bc98c3a0d",
            patches=[
                "x264-0001-Prevent-mb_info_free-to-be-called-before-all-threads.patch",
            ],
        )

    def build(self):
        configuration = (
            "debug-optimized"
            if self.opts.release_configuration_is_actually_debug_optimized
            else self.opts.configuration
        )
        msys_path = Project.get_tool_path("msys2")
        self.exec_vs(
            rf"{msys_path}\bash build\build.sh {convert_to_msys(self.builder.gtk_dir)} {configuration}",
            add_path=msys_path,
        )

        # use the path expected when building with a dependent project
        self.builder.exec_msys(
            ["mv", "libx264.dll.lib", "libx264.lib"],
            working_dir=os.path.join(self.builder.gtk_dir, "lib"),
        )

        if configuration in ["debug-optimized", "debug"]:
            self.install(r".\libx264-164.pdb bin")
            self.install(r".\x264.pdb bin")

        self.install(r".\COPYING share\doc\x264")



#  Copyright (C) 2016 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

import pytest
from typer.testing import CliRunner


@pytest.fixture
def typer_app():
    from gvsbuild.main import app

    yield app
    del app


@pytest.fixture(scope="session")
def tmp_dir(tmp_path_factory):
    return tmp_path_factory.mktemp("gtk-build")


@pytest.fixture
def runner():
    return CliRunner()



#  Copyright (C) 2016 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

"""Various builders (meson, CMake, ...) class."""

import os
import shutil
import sys
from pathlib import Path

from .base_project import Project
from .simple_ui import log
from .utils import _rmtree_error_handler


class Meson(Project):
    def __init__(self, name, **kwargs):
        Project.__init__(self, name, **kwargs)
        self._ensure_params()

    def _ensure_params(self):
        if not hasattr(self, "params"):
            self.params = []

    def add_param(self, par):
        self._ensure_params()
        self.params.append(par)

    def build(self, meson_params=None, make_tests=False, add_path=None):
        # where we build, with ninja, the library
        ninja_build = os.path.join(self.build_dir, "_gvsbuild-meson")

        # First we check if we need to generate the meson build files
        if not os.path.isfile(os.path.join(ninja_build, "build.ninja")):
            self._setup_meson_and_ninja(ninja_build, meson_params, add_path)
        if make_tests:
            # Run ninja to build all (library, ....
            self.builder.exec_ninja(working_dir=ninja_build)
            # .. run the tests ...
            self.builder.exec_ninja(params="test", working_dir=ninja_build)
            # .. and finally install everything
        # if we don't make the tests we simply run 'ninja install' that takes care of everything,
        # running explicitly from the build dir
        self.builder.exec_ninja(params="install", working_dir=ninja_build)

    def _setup_meson_and_ninja(self, ninja_build, meson_params, add_path):
        log.start_verbose("Generating meson directory")
        self.builder.make_dir(ninja_build)
        # base params
        self._ensure_params()
        add_opts = " ".join(self.params) + " " if self.params else ""
        # debug info
        build_type = self.builder.opts.configuration
        if self.builder.opts.release_configuration_is_actually_debug_optimized:
            build_type = "debugoptimized"
        add_opts += f"--buildtype {build_type}"
        if meson_params:
            add_opts += f" {meson_params}"
        if self.extra_opts:
            extra_opts = " ".join(self.extra_opts)
            add_opts += f" {extra_opts}"
        # python meson.py src_dir ninja_build_dir --prefix gtk_bin options
        meson = Project.get_tool_executable("meson")
        python = Path(sys.executable)
        if " " in str(python):
            python = f'"{python}"'
        cmd = f"{python} {meson} setup {self._get_working_dir()} {ninja_build} --prefix {self.builder.gtk_dir} {add_opts}"

        # build the ninja file to do everything (build the library, create the .pc file, install it, ...)
        self.exec_vs(cmd, add_path=add_path)
        log.end()


class CmakeProject(Project):
    def __init__(self, name, **kwargs):
        Project.__init__(self, name, **kwargs)

    def build(
        self,
        cmake_params=None,
        use_ninja=False,
        make_tests=False,
        do_install=True,
        out_of_source=None,
        source_part=None,
    ):
        cmake_gen = "Ninja" if use_ninja else "NMake Makefiles"

        cmake_config = (
            "Debug" if self.builder.opts.configuration == "debug" else "Release"
        )
        if self.builder.opts.release_configuration_is_actually_debug_optimized:
            cmake_config = "RelWithDebInfo"
        # Create the command for cmake
        cmd = f'cmake -G "{cmake_gen}" -DCMAKE_INSTALL_PREFIX="%(pkg_dir)s" -DGTK_DIR="%(gtk_dir)s" -DCMAKE_BUILD_TYPE={cmake_config}'
        if cmake_params:
            cmd += f" {cmake_params}"
        if self.extra_opts:
            extra_opts = " ".join(self.extra_opts)
            cmd += f" {extra_opts}"
        if use_ninja and out_of_source is None:
            # For ninja the default is build out of source
            out_of_source = True

        if out_of_source:
            cmake_dir = os.path.join(self.build_dir, "_gvsbuild-cmake")

            self.builder.make_dir(cmake_dir)
            if source_part:
                src_full = os.path.join(self.build_dir, source_part)
            else:
                src_full = self.build_dir
            cmd += f" -B{cmake_dir} -H{src_full}"
            work_dir = cmake_dir
        else:
            work_dir = self._get_working_dir()

        # Generate the files used to build
        log.start_verbose("Generating/updating cmake files")
        self.builder.exec_vs(cmd, working_dir=work_dir)
        log.end()
        # Build
        if use_ninja:
            if make_tests:
                self.builder.exec_ninja(working_dir=work_dir)
                self.builder.exec_ninja(params="test", working_dir=work_dir)
                if do_install:
                    self.builder.exec_ninja(params="install", working_dir=work_dir)
            elif do_install:
                self.builder.exec_ninja(params="install", working_dir=work_dir)
            else:
                self.builder.exec_ninja(working_dir=work_dir)
        else:
            self.builder.exec_vs("nmake /nologo", working_dir=work_dir)
            if do_install:
                self.builder.exec_vs("nmake /nologo install", working_dir=work_dir)


class Rust(Project):
    def __init__(self, name, **kwargs):
        Project.__init__(self, name, **kwargs)
        self._ensure_params()

    def _ensure_params(self):
        if not hasattr(self, "params"):
            self.params = []

    def add_param(self, par):
        self._ensure_params()
        self.params.append(par)

    def build(self, cargo_params=None, make_tests=False):
        rustc_opts = {}

        params = cargo_params[:] if cargo_params else []
        if self.builder.opts.configuration == "release":
            # add debug symbols anyway
            rustc_opts["RUSTFLAGS"] = "-g"
            params.append("--release")
            folder = "release"
        else:
            folder = "debug"

        if self.extra_opts:
            params.extend(self.extra_opts)

        cargo_build = os.path.join(self.build_dir, "cargo-build")

        params.append(f"--target-dir={cargo_build}")

        if self.clean and os.path.exists(cargo_build):
            log.debug(f"Removing cargo build dir '{cargo_build}'")
            shutil.rmtree(cargo_build, onerror=_rmtree_error_handler)

        # build
        self.builder.exec_cargo(
            params=" ".join(["build"] + params),
            working_dir=self.build_dir,
            rustc_opts=rustc_opts,
            rust_version=self.version,
        )

        # test
        if make_tests:
            self.builder.exec_cargo(
                params=" ".join(["test"] + params),
                working_dir=self.build_dir,
                rustc_opts=rustc_opts,
                rust_version=self.version,
            )

        shutil.copytree(
            os.path.join(cargo_build, folder), os.path.join(cargo_build, "lib")
        )


class MakeGir:
    """Class to build, with nmake, a single project .gir/.typelib files for the
    gobject-introspection support, used where the meson script is not present
    (gtk2 % gtk3) or not update the handle it."""

    def make_single_gir(self, prj_name, prj_dir=None):
        if not prj_dir:
            prj_dir = prj_name

        b_dir = f"{self.builder.working_dir}\\{prj_dir}\\build\\win32"
        if not os.path.isfile(os.path.join(b_dir, "detectenv-msvc.mak")):
            b_dir = f"{self.builder.working_dir}\\{prj_dir}\\win32"
            if not os.path.isfile(os.path.join(b_dir, "detectenv-msvc.mak")):
                log.message(f"Unable to find detectenv-msvc.mak for {prj_name}")
                return

        cmd = f"nmake -f {prj_name}-introspection-msvc.mak CFG={self.builder.opts.configuration} PREFIX={self.builder.gtk_dir} PYTHON={Project.get_tool_executable('python')} install-introspection"

        self.push_location(b_dir)
        self.exec_vs(cmd)
        self.pop_location()



#  Copyright (C) 2016 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

from gvsbuild.utils.base_builders import Meson
from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import project_add


@project_add
class Graphene(Tarball, Meson):
    def __init__(self):
        Meson.__init__(
            self,
            "graphene",
            version="1.10.8",
            repository="ebassi/graphene",
            archive_url="https://github.com/ebassi/graphene/archive/refs/tags/{version}.tar.gz",
            archive_filename="graphene-{version}.tar.gz",
            hash="922dc109d2dc5dc56617a29bd716c79dd84db31721a8493a13a5f79109a4a4ed",
            dependencies=["ninja", "meson", "pkgconf", "glib"],
            patches=["001-fix-python-lookup.patch"],
        )
        if self.opts.enable_gi:
            self.add_dependency("gobject-introspection")
            enable_gi = "enabled"
        else:
            enable_gi = "disabled"

        self.add_param(f"-Dintrospection={enable_gi}")

    def build(self):
        Meson.build(self, make_tests=True)
        self.install(r".\LICENSE share\doc\graphene")



#!/usr/bin/env python3
#
# gperf version.py
#
# Copyright (C) 2020 Tim-Philipp Müller <tim centricular com>
#
# This file is part of GNU GPERF.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# Extracts version from src/version.cc, because like any sane build system
# we extract the version from the code instead of having the build system set
# the version for the code to consume.

import os
import sys

version = None

srcroot = os.path.dirname(__file__)

f = open(os.path.join(srcroot, "src/version.cc"))

for line in f:
    if line.startswith("const char *version_string"):
        version = line[26:].split('"')[1]
        break

f.close()

if not version:
    print(
        "Warning: Could not extract version from src/version.cc in",
        srcroot,
        file=sys.stderr,
    )
    sys.exit(-1)

print(version)
sys.exit(0)



#  Copyright (C) 2016 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

"""Simple user interface for info, log & debug messages."""

import ctypes
import datetime
import os
import sys
from contextlib import contextmanager, suppress

from rich import print

# Original windows console title
_script_org_title = None


def script_title(new_title):
    """Set the new console title for the running script, saving the old one.

    Passing None to the title restores the old, saved, one
    """

    global _script_org_title
    if new_title:
        # Save the old title
        if _script_org_title is None:
            buf = ctypes.create_unicode_buffer(256)
            ctypes.windll.kernel32.GetConsoleTitleW(buf, 256)
            _script_org_title = buf.value
        ctypes.windll.kernel32.SetConsoleTitleW(f"gvsbuild {new_title}")
    elif _script_org_title is not None:
        # Restore old title
        ctypes.windll.kernel32.SetConsoleTitleW(_script_org_title)
        # cleanup if we want to call the function again
        _script_org_title = None


# Log levels
LOG_ALWAYS = 1
LOG_VERBOSE = 2
LOG_DEBUG = 3


class LogElem:
    def __init__(self, msg, enabled, tim=None):
        self.msg = msg
        self.tim = tim or datetime.datetime.now()
        self.indent = not enabled
        self.enabled = enabled


class Log:
    """Simple log class, used mainly to time the execution of the script."""

    _verbose = False
    _debug = False

    def __init__(self):
        self.st_time = datetime.datetime.now()
        self.fo = None
        self.level = LOG_ALWAYS

    def configure(self, file_path, opts=None):
        max_size_kb = 0
        single = False
        if opts:
            if opts.debug:
                self._verbose = True
                self._debug = True
                self.level = LOG_DEBUG
            elif opts.verbose:
                self._verbose = True
                self.level = LOG_VERBOSE
            single = opts.log_single
            self.capture = opts.capture_out
            max_size_kb = 0 if single else opts.log_size
        if file_path:
            self._create_log(file_path, single, max_size_kb, opts)

    def _create_log(self, file_path, single, max_size_kb, opts):
        if not os.path.exists(file_path):
            created = True
            os.makedirs(file_path)
        else:
            created = False

        file_name = (
            self.st_time.strftime("gvsbuild-log-%Y%m%d-%H%M%S.txt")
            if single
            else "gvsbuild-log.txt"
        )

        self.log_file = os.path.join(file_path, file_name)
        if max_size_kb:
            try:
                c_size = os.path.getsize(self.log_file) / 1024
            except Exception as e:
                print("Exception reading log file size (%s)", self.log_file)
                print(e)
                c_size = 0

            if c_size > max_size_kb:
                old_file = os.path.join(file_path, "gvsbuild-log.old.txt")
                with suppress(FileNotFoundError):
                    os.remove(old_file)
                os.rename(self.log_file, old_file)

        self.operations = []
        self.fo = open(self.log_file, "a", encoding="utf-8")
        self._output("Script started")
        if created:
            self.log(f"Log directory {file_path} created")
        if opts and not self._debug:
            # Dump some information
            self._output_val("Configuration", opts.configuration)
            self._output_val("Platform", opts.platform)
            self._output_val("Vs ver", opts.vs_ver)
            self._output_val("Vs path", opts.vs_install_path)
            self._output_val("Sdk ver", opts.win_sdk_ver)

    def _get_delta(self, start):
        dt = datetime.datetime.now() - start
        return f"{dt.seconds}.{dt.microseconds // 1000:03}"

    def _indend_check(self):
        if self.operations:
            co = self.operations[-1]
            if not co.indent:
                # not yet logged
                self.operations.pop()
                self._output(f"{co.msg} ...", check_indent=False)
                co.indent = True
                self.operations.append(co)

    def close(self):
        while self.operations:
            self.end()

        self._output(f"Script ended correctly ({self._get_delta(self.st_time)} s)")
        # The \n is correct, to separate other build's logs
        self._output("--------\n")
        self.fo.close()
        self.fo = None

    def start(self, msg, level=LOG_ALWAYS):
        enabled = level <= self.level
        if enabled:
            print(msg)
            self._indend_check()

        if self.capture:
            self._output(msg, check_indent=False)

        co = LogElem(msg, enabled)
        self.operations.append(co)

    def start_verbose(self, msg):
        self.start(msg, level=LOG_VERBOSE)

    def start_debug(self, msg):
        self.start(msg, level=LOG_DEBUG)

    def end(self, force_print=False, mark_error=False):
        if self.operations:
            co = self.operations.pop()
            if co.enabled:
                if mark_error:
                    out_msg = f"*** Error: {co.msg} ({self._get_delta(co.tim)} s)"
                else:
                    out_msg = f"{co.msg} - Ended in {self._get_delta(co.tim)} s"
                self._output(out_msg, check_indent=False)
                if force_print:
                    print(out_msg)
                if not self.operations:
                    self.flush()

    def flush(self):
        if self.fo:
            self.fo.flush()

    def _output(self, msg, add_date=True, check_indent=True):
        if self.fo:
            if check_indent:
                self._indend_check()
            msg = " " * len(self.operations) * 2 + msg
            if add_date:
                now_val = datetime.datetime.now()
                self.fo.write(f"{now_val.strftime('%Y-%m-%d %H:%M:%S')} {msg}\n")
            else:
                self.fo.write(f"{msg:<20}")
            return False
        else:
            print(msg)
            # tell the caller we already print on video
            return True

    def _output_val(self, msg, val):
        self._output(f"{msg:<16}: {val}")

    def message_indent(self, msg):
        if self._output(msg, add_date=False):
            return
        print(f"  {msg}")

    def message(self, msg):
        if self._output(msg):
            # already printed
            return
        print(msg)

    """
    Like message(...) but if the log capture is active is not printed on screen
    """

    def info(self, msg):
        if self._output(msg):
            return
        if not self.capture:
            print(msg)

    def messages_dump(self, msgs, prt=False, err=None):
        if err is not None:
            if not err:
                err = "Attention! Error presents!"
            self.message(err)
            self.message("")
            # with error we want to know what's happeninh
            prt = True

        lines = msgs.split("\n")
        lines = [line.rstrip() for line in lines if line.rstrip() != ""]
        if self.fo:
            # On the file, if active
            for line in lines:
                self.fo.write(f"    {line}\n")

        if prt:
            for line in lines:
                print(line)

    def log(self, msg):
        if self._verbose:
            if self._output(msg):
                return
            print(msg)
        elif self.fo:
            self._output(msg)

    def debug(self, msg):
        if self._debug:
            if self._output(msg):
                return
            print("Debug:", msg)
        elif self.fo:
            self._output(msg)

    def verbose_on(self):
        return self._verbose

    def debug_on(self):
        return self._debug

    def error_exit(self, msg):
        self._output(f"Error:{msg}")
        print("Error:", msg, file=sys.stderr)
        sys.exit(1)

    @staticmethod
    @contextmanager
    def simple_oper(msg, level=LOG_ALWAYS):
        """To time single operation, using: with log.simple_oper('timing ...'):

        .. do stuff
        """
        log.start(msg, level)
        yield True
        log.end()


# single instance of the log class
log = Log()

if __name__ == "__main__":
    import time

    log.configure(".", None)
    log.start("Test #1")
    time.sleep(0.8)
    log.start("Test #2, nested")
    time.sleep(0.5)
    log.end()
    with log.simple_oper("Test with context manager"):
        time.sleep(0.35)
        with log.simple_oper("Second test with context manager"):
            time.sleep(0.15)
    log.close()



#  Copyright (C) 2016 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

import os

from gvsbuild.utils.base_builders import CmakeProject
from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class Libfido2(Tarball, CmakeProject):
    def __init__(self):
        Project.__init__(
            self,
            "libfido2",
            version="1.15.0",
            archive_url="https://developers.yubico.com/libfido2/Releases/libfido2-{version}.tar.gz",
            hash="abaab1318d21d262ece416fb8a7132fa9374bda89f6fa52b86a98a2f5712b61e",
            dependencies=[
                "cmake",
                "ninja",
                "zlib",
                "openssl",
                "libcbor",
            ],
            patches=[
                "0001-libfido2-update-cmake-script-to-have-sdl-flag-before.patch",
            ],
        )

    def build(self):
        if self.builder.x86:
            arch = "x86"
        else:
            arch = "x64"

        include_dirs = os.path.join(self.builder.gtk_dir, "inc")
        lib_dirs = os.path.join(self.builder.gtk_dir, "lib")
        bin_dirs = lib_dirs = os.path.join(self.builder.gtk_dir, "bin")

        build_params = "-DBUILD_EXAMPLES=OFF -DBUILD_MANPAGES=OFF -DBUILD_TESTS=OFF -DBUILD_TOOLS=OFF -DBUILD_STATIC_LIBS=OFF"
        cmake_params = f"-DWITH_ZLIB=ON -DCBOR_INCLUDE_DIRS={include_dirs} -DCRYPTO_INCLUDE_DIRS={include_dirs} -DZLIB_INCLUDE_DIRS={include_dirs} -DCBOR_LIBRARY_DIRS={lib_dirs} -DCRYPTO_LIBRARY_DIRS={lib_dirs} -DZLIB_LIBRARY_DIRS={lib_dirs} -DCBOR_BIN_DIRS={bin_dirs} -DCRYPTO_BIN_DIRS={bin_dirs} -DZLIB_BIN_DIRS={bin_dirs} -DCRYPTO_LIBRARIES=libcrypto {build_params}"

        CmakeProject.build(self, cmake_params=cmake_params, use_ninja=True)
        self.install(rf"output\{arch}\static\* .")



#  Copyright (C) 2016 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

from gvsbuild.utils.base_builders import CmakeProject
from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class Protobuf(Tarball, CmakeProject):
    def __init__(self):
        Project.__init__(
            self,
            "protobuf",
            version="25.1",
            archive_url="https://github.com/protocolbuffers/protobuf/releases/download/v{version}/protobuf-{version}.tar.gz",
            hash="9bd87b8280ef720d3240514f884e56a712f2218f0d693b48050c836028940a42",
            dependencies=[
                "abseil-cpp",
                "cmake",
                "zlib",
                "ninja",
            ],
        )

    def build(self):
        # We need to compile with STATIC_RUNTIME off since protobuf-c also compiles with it OFF
        CmakeProject.build(
            self,
            cmake_params=r'-DBUILD_SHARED_LIBS=ON -Dprotobuf_ABSL_PROVIDER=package -Dprotobuf_DEBUG_POSTFIX="" -Dprotobuf_BUILD_TESTS=OFF -Dprotobuf_WITH_ZLIB=ON -Dprotobuf_MSVC_STATIC_RUNTIME=OFF',
            use_ninja=True,
        )

        self.install(r".\LICENSE share\doc\protobuf")


@project_add
class ProtobufC(Tarball, CmakeProject):
    def __init__(self):
        Project.__init__(
            self,
            "protobuf-c",
            version="1.5.1",
            archive_url="https://github.com/protobuf-c/protobuf-c/releases/download/v{version}/protobuf-c-{version}.tar.gz",
            hash="20d1dc257da96f8ddff8be4dd9779215bbd0a6069ed53bbe9de38fa7629be06b",
            dependencies=[
                "abseil-cpp",
                "cmake",
                "protobuf",
                "ninja",
            ],
            patches=[
                "0001-CMakeList.txt-Remove-double-dashes.patch",
            ],
        )

    def build(self):
        CmakeProject.build(
            self,
            cmake_params="-DBUILD_SHARED_LIBS=ON",
            use_ninja=True,
            source_part="build-cmake",
        )

        self.install(r".\LICENSE share\doc\protobuf-c")
        self.install_pc_files()



#  Copyright (C) 2016 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

import os

from gvsbuild.utils.base_builders import Meson
from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class GStreamer(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "gstreamer",
            repository="https://gitlab.freedesktop.org/gstreamer/gstreamer",
            version="1.26.0",
            lastversion_even=True,
            archive_url="https://gstreamer.freedesktop.org/src/gstreamer/gstreamer-{version}.tar.xz",
            hash="1b2ee4028010c25b776effa7c396c7e3e1861b60b9417e416f4914abcdff279f",
            dependencies=["meson", "ninja", "glib", "orc"],
            patches=[],
        )

        if self.opts.enable_gi:
            self.add_dependency("gobject-introspection")
            enable_gi = "enabled"
        else:
            enable_gi = "disabled"

        self.add_param(f"-Dintrospection={enable_gi}")

    def build(self):
        add_path = os.path.join(self.builder.opts.msys_dir, "usr", "bin")

        Meson.build(
            self, add_path=add_path, meson_params="-Dtests=disabled -Dexamples=disabled"
        )
        self.install(r".\COPYING share\doc\gstreamer")


@project_add
class Orc(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "orc",
            version="0.4.41",
            lastversion_even=True,
            repository="https://gitlab.freedesktop.org/gstreamer/orc",
            archive_url="https://gstreamer.freedesktop.org/src/orc/orc-{version}.tar.xz",
            hash="cb1bfd4f655289cd39bc04642d597be9de5427623f0861c1fc19c08d98467fa2",
            dependencies=["meson", "ninja"],
        )

    def build(self):
        Meson.build(self, meson_params="-Dbenchmarks=disabled -Dtools=enabled")
        self.install(r"COPYING share\doc\orc")


@project_add
class GstPluginsBase(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "gst-plugins-base",
            repository="https://gitlab.freedesktop.org/gstreamer/gstreamer",
            version="1.26.0",
            lastversion_even=True,
            archive_url="https://gstreamer.freedesktop.org/src/gst-plugins-base/gst-plugins-base-{version}.tar.xz",
            hash="e23189fbed2ec486690382d1055c19eeaf5aae3e95e2576fc4c884d96a90e69e",
            dependencies=[
                "meson",
                "ninja",
                "gstreamer",
                "opus",
                "ogg",
                "graphene",
                "pango",
            ],
        )
        # Examples depend on GTK3
        self.add_param("-Dexamples=disabled")

        if self.opts.enable_gi:
            self.add_dependency("gobject-introspection")
            enable_gi = "enabled"
        else:
            enable_gi = "disabled"

        self.add_param(f"-Dintrospection={enable_gi}")

    def build(self):
        Meson.build(
            self, meson_params=f'-Dc_link_args="{self.builder.gtk_dir}\\lib\\ogg.lib"'
        )
        self.install(r".\COPYING share\doc\gst-plugins-base")


@project_add
class GstPluginsGood(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "gst-plugins-good",
            repository="https://gitlab.freedesktop.org/gstreamer/gstreamer",
            version="1.26.0",
            lastversion_even=True,
            archive_url="https://gstreamer.freedesktop.org/src/gst-plugins-good/gst-plugins-good-{version}.tar.xz",
            hash="9e18f139ef69ad0867c2dfbb8fe1d1736d76df11aac83f60e8d3adb1e2eaf03b",
            dependencies=[
                "meson",
                "ninja",
                "gst-plugins-base",
                "libvpx",
            ],
        )

    def build(self):
        Meson.build(self)
        self.install(r".\COPYING share\doc\gst-plugins-good")


@project_add
class GstPluginsBad(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "gst-plugins-bad",
            repository="https://gitlab.freedesktop.org/gstreamer/gstreamer",
            version="1.26.0",
            lastversion_even=True,
            archive_url="https://gstreamer.freedesktop.org/src/gst-plugins-bad/gst-plugins-bad-{version}.tar.xz",
            hash="f8287a84c5f66368a5a50da5f969994a02c47f20220ffe1ca3154193e65af216",
            dependencies=[
                "meson",
                "ninja",
                "gst-plugins-base",
                "libnice",
                "webrtc-audio-processing",
                "openssl",
                "libsrtp2",
            ],
            patches=[
                "wasapisink-reduce-buffer-latency.patch",
            ],
        )
        self.add_param("-Dcurl=disabled")
        self.add_param("-Dcurl-ssh2=disabled")

        # Adding webrtc-audio-processing adds the isac plugin. It fails to compile.
        self.add_param("-Disac=disabled")

        if self.opts.enable_gi:
            self.add_dependency("gobject-introspection")
            enable_gi = "enabled"
        else:
            enable_gi = "disabled"

        self.add_param(f"-Dintrospection={enable_gi}")

    def build(self):
        Meson.build(self)
        self.install(r".\COPYING share\doc\gst-plugins-bad")


@project_add
class GstPluginsUgly(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "gst-plugins-ugly",
            repository="https://gitlab.freedesktop.org/gstreamer/gstreamer",
            version="1.26.0",
            lastversion_even=True,
            archive_url="https://gstreamer.freedesktop.org/src/gst-plugins-ugly/gst-plugins-ugly-{version}.tar.xz",
            hash="a86b51c8454a813120848c803421f327d8c07aabcae461e0597cc49398c0fcde",
            dependencies=["meson", "ninja", "gst-plugins-base"],
        )

    def build(self):
        Meson.build(self)
        self.install(r".\COPYING share\doc\gst-plugins-ugly")


@project_add
class GstDevTools(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "gst-devtools",
            repository="https://gitlab.freedesktop.org/gstreamer/gstreamer",
            version="1.26.0",
            lastversion_even=True,
            archive_url="https://gstreamer.freedesktop.org/src/gst-devtools/gst-devtools-{version}.tar.xz",
            hash="eff33d7dc292bb074a2788ea887b6283398ffdefafa49fb7d08efe6658a65648",
            dependencies=[
                "meson",
                "ninja",
                "json-glib",
                "gst-plugins-base",
                "gst-plugins-bad",
                "gst-rtsp-server",
            ],
        )

        if self.opts.enable_gi:
            self.add_dependency("gobject-introspection")
            enable_gi = "enabled"
        else:
            enable_gi = "disabled"

        self.add_param(f"-Dintrospection={enable_gi}")

    def build(self):
        Meson.build(self)
        self.install(r".\COPYING share\doc\gst-devtools")


@project_add
class GstPython(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "gst-python",
            repository="https://gitlab.freedesktop.org/gstreamer/gstreamer",
            version="1.26.0",
            lastversion_even=True,
            archive_url="https://gstreamer.freedesktop.org/src/gst-python/gst-python-{version}.tar.xz",
            hash="e5046a05d77abb15671ad01cd1908d17d62e5a06f5d7841be4342ade2a3fb8db",
            dependencies=[
                "meson",
                "ninja",
                "pygobject",
                "gst-plugins-base",
                "gst-plugins-bad",
            ],
        )

    def build(self):
        Meson.build(self)
        self.install(r".\COPYING share\doc\gst-python")


@project_add
class GstLibav(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "gst-libav",
            repository="https://gitlab.freedesktop.org/gstreamer/gstreamer",
            version="1.26.0",
            lastversion_even=True,
            archive_url="https://gstreamer.freedesktop.org/src/gst-libav/gst-libav-{version}.tar.xz",
            hash="707a8b687ff5fddcee5b02415e2ec9b71b4ac44d0b7aec3b477364ceecbf1ecf",
            # TODO try remove gst-plugins-base
            dependencies=["meson", "ninja", "pygobject", "ffmpeg", "gst-plugins-base"],
        )

    def build(self):
        Meson.build(self)
        self.install(r".\COPYING share\doc\gst-libav")


@project_add
class GstRtspServer(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "gst-rtsp-server",
            repository="https://gitlab.freedesktop.org/gstreamer/gstreamer",
            version="1.26.0",
            lastversion_even=True,
            archive_url="https://gstreamer.freedesktop.org/src/gst-rtsp-server/gst-rtsp-server-{version}.tar.xz",
            hash="e983c039496e3f75e39696554ce74db4120e2465de17aa1cc37160568e9b40bc",
            dependencies=["meson", "ninja", "gstreamer"],
        )

    def build(self):
        Meson.build(self)
        self.install(r".\COPYING share\doc\gst-rtsp-server")



#  Copyright (C) 2016 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

import os

from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class Kerberos(Tarball, Project):
    def __init__(self):
        Project.__init__(
            self,
            "mit-kerberos",
            version="1.21.3",
            repository="https://github.com/krb5/krb5",
            archive_url="https://github.com/krb5/krb5/archive/refs/tags/krb5-{version}-final.tar.gz",
            hash="2157d92020d408ed63ebcd886a92d1346a1383b0f91123a0473b4f69b4a24861",
            dependencies=[
                "perl",
            ],
        )

    def build(self):
        configuration = (
            "Debug" if self.builder.opts.configuration == "debug" else "Release"
        )
        add_path = os.path.join(self.builder.opts.msys_dir, "usr", "bin")

        self.push_location("src")
        self.exec_vs(
            r"nmake -f Makefile.in prep-windows NO_LEASH=1 KRB_INSTALL_DIR=%(gtk_dir)s ",
            add_path=add_path,
        )
        self.exec_vs(
            r"nmake NODEBUG="
            + str(1 if configuration == "Release" else 0)
            + " NO_LEASH=1 KRB_INSTALL_DIR=%(gtk_dir)s ",
            add_path=add_path,
        )
        self.exec_vs(
            r"nmake install NODEBUG="
            + str(1 if configuration == "Release" else 0)
            + " NO_LEASH=1 KRB_INSTALL_DIR=%(gtk_dir)s ",
            add_path=add_path,
        )
        self.pop_location()

        self.install(r".\NOTICE share\doc\mit-kerberos")



#  Copyright (C) 2016 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

from gvsbuild.utils.base_builders import Meson
from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class Libxml2(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "libxml2",
            version="2.12.10",
            lastversion_even=True,
            repository="https://gitlab.gnome.org/GNOME/libxml2",
            archive_url="https://download.gnome.org/sources/libxml2/{major}.{minor}/libxml2-{version}.tar.xz",
            hash="c3d8c0c34aa39098f66576fe51969db12a5100b956233dc56506f7a8679be995",
            dependencies=["win-iconv", "meson", "ninja"],
            patches=[],
        )

    def build(self):
        Meson.build(self)
        self.install_pc_files()
        self.install(r".\COPYING share\doc\libxml2")



#  Copyright (C) 2016 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

import os

from gvsbuild.utils.base_builders import Meson
from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import project_add


@project_add
class AdwaitaIconTheme(Tarball, Meson):
    def __init__(self):
        Meson.__init__(
            self,
            "adwaita-icon-theme",
            version="48.0",
            repository="https://gitlab.gnome.org/GNOME/adwaita-icon-theme",
            archive_url="https://download.gnome.org/sources/adwaita-icon-theme/{major}/adwaita-icon-theme-{version}.tar.xz",
            hash="847068888650d9673115be6dbf2bfdc31a46aebc528a6a9db4420e60e656b8d4",
            dependencies=[
                "hicolor-icon-theme",
                "librsvg",
            ],
        )

    def build(self):
        Meson.build(self)
        # Work around for https://gitlab.gnome.org/GNOME/adwaita-icon-theme/-/issues/282
        self.builder.exec_msys(
            [
                "cp",
                "--remove-destination",
                "places/folder.svg",
                "status/folder-open.svg",
            ],
            working_dir=os.path.join(
                self.builder.gtk_dir, "share", "icons", "Adwaita", "scalable"
            ),
        )
        self.install(r".\COPYING_CCBYSA3 share\doc\adwaita-icon-theme")



#  Copyright (C) 2016 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

from gvsbuild.utils.base_builders import Meson
from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import project_add


@project_add
class Cairomm(Tarball, Meson):
    def __init__(self):
        Meson.__init__(
            self,
            "cairomm",
            version="1.18.0",
            repository="https://gitlab.freedesktop.org/cairo/cairomm",
            archive_url="https://cairographics.org/releases/cairomm-{version}.tar.xz",
            hash="b81255394e3ea8e8aa887276d22afa8985fc8daef60692eb2407d23049f03cfb",
            dependencies=["meson", "ninja", "libsigc++", "cairo"],
        )

    def build(self):
        Meson.build(
            self, meson_params="-Dbuild-examples=false -Dbuild-documentation=false"
        )
        self.install(r".\COPYING share\doc\cairomm")



#  Copyright (C) 2016 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

import json

import typer

from gvsbuild.utils.base_project import Project, ProjectType


def list_(
    projects_names: list[str] = typer.Argument(None, help="The projects to list"),
    project_type: ProjectType = typer.Option(
        None,
        "--type",
        help="Specify type of projects to show, if not selected show all",
        rich_help_panel="Selection Options",
    ),
    show_deps: bool = typer.Option(
        False,
        "--deps",
        help="Include dependencies, only useful when selecting the projects to show",
        rich_help_panel="Selection Options",
    ),
    json_: bool = typer.Option(
        False,
        "--json",
        help="Show list in JSON format",
        rich_help_panel="Formatting Options",
    ),
):
    Project.add_all()

    projects = Project.list_projects()
    if projects_names:
        projects = [project for project in projects if project.name in projects_names]
    if show_deps:
        projects = Project.compute_dependencies(projects)
    if project_type:
        projects = [
            project for project in projects if project.type.value == project_type
        ]

    if json_:

        def _get_project_data(project):
            data = {"dependencies": project.dependencies, "type": project.type.value}
            if project.version:
                data["version"] = project.version
            if hasattr(project, "latest_version") and project.latest_version:
                data["latest-version"] = str(project.latest_version)
            return data

        print(
            json.dumps(
                {project.name: _get_project_data(project) for project in projects},
                indent=4,
                sort_keys=True,
            )
        )
    else:
        for project_type in ProjectType:
            type_projects = [
                project for project in projects if project.type == project_type
            ]
            if type_projects:
                print(f"Available projects with type {project_type}:")
                for project in type_projects:
                    params = {
                        "name": f"{project.name:<{Project.name_len}}",
                        "version": f"{project.version:<45}",
                    }
                    print("\t{name} {version}".format(**params))



#  Copyright (C) 2016 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

from gvsbuild.utils.base_builders import Meson
from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class Pixman(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "pixman",
            repository="https://gitlab.freedesktop.org/pixman/pixman",
            version="0.44.2",
            archive_url="http://cairographics.org/releases/pixman-{version}.tar.gz",
            hash="6349061ce1a338ab6952b92194d1b0377472244208d47ff25bef86fc71973466",
            dependencies=["ninja", "meson"],
        )

    def build(self):
        enable_mmx = "disabled" if self.builder.x64 else "enabled"
        Meson.build(
            self,
            meson_params=f"-Dsse2=enabled -Dssse3=enabled -Dmmx={enable_mmx} -Dtests=disabled",
        )

        self.install(r".\COPYING share\doc\pixman")



#  Copyright (C) 2025 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.
from __future__ import annotations

import sys
import tarfile
import zipfile
from collections.abc import Iterator
from contextlib import contextmanager
from io import BytesIO
from pathlib import Path
from tempfile import TemporaryDirectory
from unittest.mock import Mock, patch

import pytest

from gvsbuild.utils.base_expanders import (
    _get_stripped_tar_members,
    _is_safe_link_target,
    _is_unsafe_path,
    _is_within_directory,
    _safe_extractall,
    _strip_path,
    extract_exec,
)


@contextmanager
def create_test_tar(
    files: dict[str, bytes | None], symlinks: dict[str, str] | None = None
) -> Iterator[tarfile.TarFile]:
    """
    Create a test tar file with specified files and symlinks.
    """
    tar_buffer = BytesIO()
    with tarfile.open(fileobj=tar_buffer, mode="w:gz") as tar:
        if files or symlinks:
            # Create base directory only if we have content
            base_info = tarfile.TarInfo("testdir")
            base_info.type = tarfile.DIRTYPE
            tar.addfile(base_info)

        # Add files
        for path, content in files.items():
            path = Path(path).as_posix()
            info = tarfile.TarInfo(f"testdir/{path}")
            if content is None:
                info.type = tarfile.DIRTYPE
            else:
                info.type = tarfile.REGTYPE
                info.size = len(content)
                content_buffer = BytesIO(content)
                tar.addfile(info, content_buffer)

        # Add symlinks
        if symlinks:
            for link_path, target in symlinks.items():
                link_path = Path(link_path).as_posix()
                target = Path(target).as_posix()
                info = tarfile.TarInfo(f"testdir/{link_path}")
                info.type = tarfile.SYMTYPE
                info.linkname = target
                tar.addfile(info)

    tar_buffer.seek(0)
    yield tarfile.open(fileobj=tar_buffer, mode="r:gz")


@pytest.fixture
def temp_workspace():
    """Create a temporary workspace with test files."""
    with TemporaryDirectory() as temp_dir:
        temp_path = Path(temp_dir)

        # Create test exe
        exe_path = temp_path / "test.exe"
        exe_path.write_bytes(b"fake exe content")

        # Create test zip with base directory
        zip_path = temp_path / "test.zip"
        with zipfile.ZipFile(zip_path, "w") as zf:
            zf.writestr("basedir/file1.txt", "content1")
            zf.writestr("basedir/subdir/file2.txt", "content2")

        # Create test tar with base directory
        tar_path = temp_path / "test.tar.gz"
        with tarfile.open(tar_path, "w:gz") as tar:
            base_info = tarfile.TarInfo("basedir")
            base_info.type = tarfile.DIRTYPE
            tar.addfile(base_info)

            info1 = tarfile.TarInfo("basedir/file1.txt")
            info1.size = 8
            tar.addfile(info1, BytesIO(b"content1"))

            info2 = tarfile.TarInfo("basedir/subdir/file2.txt")
            info2.size = 8
            tar.addfile(info2, BytesIO(b"content2"))

        yield temp_path


# Unit Tests: Path Manipulation
def test_strip_path():
    """Test the path stripping functionality."""
    assert _strip_path("testdir/file.txt") == "file.txt"


def test_strip_path_with_nested_directory():
    """Test stripping path with nested directories."""
    assert _strip_path("testdir/subdir/file.txt") == "subdir/file.txt"


def test_strip_path_without_directory():
    """Test stripping path without directory prefix."""
    assert _strip_path("file.txt") is None


def test_strip_path_base_directory():
    """Test stripping the base directory itself."""
    assert _strip_path("testdir") is None


# Unit Tests: Path Safety
def test_safe_paths():
    """Test identification of safe paths."""
    assert not _is_unsafe_path("file.txt")
    assert not _is_unsafe_path("dir/file.txt")
    assert not _is_unsafe_path("dir/subdir/file.txt")


def test_unsafe_windows_absolute_path():
    """Test identification of unsafe Windows absolute paths."""
    assert _is_unsafe_path("C:/file.txt")
    assert _is_unsafe_path("D:\\file.txt")


def test_unsafe_directory_traversal():
    """Test identification of unsafe directory traversal."""
    assert _is_unsafe_path("../file.txt")
    assert _is_unsafe_path("dir/../file.txt")
    assert _is_unsafe_path("dir/../../file.txt")


# Unit Tests: Directory Containment
def test_is_within_directory_valid_path():
    base = Path("/test/base")
    assert _is_within_directory(base / "file.txt", base) is True


def test_is_within_directory_valid_subdir():
    base = Path("/test/base")
    assert _is_within_directory(base / "subdir" / "file.txt", base) is True


def test_is_within_directory_parent_traversal():
    base = Path("/test/base")
    assert _is_within_directory(base / ".." / "file.txt", base) is False


def test_is_within_directory_deep_traversal():
    base = Path("/test/base")
    assert (
        _is_within_directory(base / "subdir" / ".." / ".." / "file.txt", base) is False
    )


def test_is_within_directory_absolute_path():
    base = Path("/test/base")
    assert _is_within_directory(Path("/other/path/file.txt"), base) is False


def test_is_within_directory_same_directory():
    base = Path("/test/base")
    assert _is_within_directory(base, base) is True


# Unit Tests: Link Safety
def test_safe_link_target_to_existing_file():
    """Test link targeting an existing file."""
    seen_files = {"file1.txt", "dir/file2.txt"}
    assert _is_safe_link_target("file1.txt", seen_files, "link.txt")


def test_safe_link_target_to_nested_file():
    """Test link targeting an existing nested file."""
    seen_files = {"file1.txt", "dir/file2.txt"}
    assert _is_safe_link_target("dir/file2.txt", seen_files, "link.txt")


def test_unsafe_link_target_directory_traversal():
    """Test link with directory traversal."""
    seen_files = {"file1.txt"}
    assert not _is_safe_link_target("../file.txt", seen_files, "link.txt")


def test_unsafe_link_target_self_reference():
    """Test self-referential link."""
    seen_files = {"file1.txt"}
    assert not _is_safe_link_target("file1.txt", seen_files, "file1.txt")


def test_unsafe_link_target_nonexistent():
    """Test link to nonexistent file."""
    seen_files = {"file1.txt"}
    assert not _is_safe_link_target("nonexistent.txt", seen_files, "link.txt")


# Unit Tests: Safe Extractall
def test_safe_extractall_creates_directory():
    """Test directory creation in safe_extractall."""
    tar_mock = Mock(spec=tarfile.TarFile)
    tar_mock.getmembers.return_value = []
    dest = Path("test_dir")

    with patch.object(Path, "mkdir") as mkdir_mock:
        _safe_extractall(tar_mock, dest)
        mkdir_mock.assert_called_once_with(parents=True, exist_ok=True)


def test_safe_extractall_blocks_directory_traversal():
    """Test that _safe_extractall prevents directory traversal via file paths."""
    tar_mock = Mock(spec=tarfile.TarFile)
    escaping_file = Mock(spec=tarfile.TarInfo)
    escaping_file.name = "../outside.txt"
    tar_mock.getmembers.return_value = [escaping_file]

    extracted_files = []

    def mock_extract(member, path):
        extracted_files.append(member.name)

    tar_mock.extract = mock_extract

    _safe_extractall(tar_mock, "test_dir")
    assert "../outside.txt" not in extracted_files


# Integration Tests: Tar Member Processing
def test_tar_extraction_single_file():
    """Test extracting a single file from tar."""
    files = {"file1.txt": b"content1"}
    with create_test_tar(files) as tar:
        members = list(_get_stripped_tar_members(tar))
        paths = {Path(m.name).as_posix() for m in members}
        assert paths == {"file1.txt"}


def test_tar_extraction_nested_files():
    """Test extracting nested files from tar."""
    files = {"subdir/file2.txt": b"content2", "subdir/": None}
    with create_test_tar(files) as tar:
        members = list(_get_stripped_tar_members(tar))
        paths = {Path(m.name).as_posix() for m in members}
        assert paths == {"subdir/file2.txt"}


def test_empty_tar_raises_error():
    """Test that empty tar raises appropriate error."""
    with create_test_tar({}) as tar:
        with pytest.raises(NotADirectoryError, match="Empty archive"):
            list(_get_stripped_tar_members(tar))


def test_invalid_tar_structure_raises_error():
    """Test that invalid tar structure raises appropriate error."""
    tar_buffer = BytesIO()
    with tarfile.open(fileobj=tar_buffer, mode="w:gz") as tar:
        info = tarfile.TarInfo("file.txt")
        info.type = tarfile.REGTYPE
        info.size = 0
        tar.addfile(info)

    tar_buffer.seek(0)
    with tarfile.open(fileobj=tar_buffer, mode="r:gz") as tar:
        with pytest.raises(NotADirectoryError, match="Cannot strip directory prefix"):
            list(_get_stripped_tar_members(tar))


# Integration Tests: Symlink Processing
def test_regular_symlink_processing():
    """Test handling of a regular symlink."""
    files = {"file1.txt": b"target content"}
    symlinks = {"link1.txt": "file1.txt"}

    with create_test_tar(files, symlinks) as tar:
        members = list(_get_stripped_tar_members(tar))
        links = {Path(m.name).as_posix(): m for m in members if m.islnk() or m.issym()}
        assert "link1.txt" in links
        assert Path(links["link1.txt"].linkname).as_posix() == "file1.txt"


def test_unsafe_symlink_conversion():
    """Test handling of symlink with parent reference."""
    files = {"file1.txt": b"content"}
    symlinks = {"link2.txt": "../file1.txt"}

    with create_test_tar(files, symlinks) as tar:
        members = list(_get_stripped_tar_members(tar))
        regular_files = {Path(m.name).as_posix(): m for m in members if m.isfile()}
        assert "link2.txt" in regular_files
        assert regular_files["link2.txt"].type == tarfile.REGTYPE


def test_circular_symlink_handling():
    """Test handling of circular symlink reference."""
    files = {}
    symlinks = {"circular.txt": "circular.txt"}

    with create_test_tar(files, symlinks) as tar:
        members = list(_get_stripped_tar_members(tar))
        regular_files = {Path(m.name).as_posix(): m for m in members if m.isfile()}
        assert "circular.txt" in regular_files
        assert regular_files["circular.txt"].type == tarfile.REGTYPE


# Integration Tests: Complex Scenarios
def test_symlink_chain_safety():
    """Test safety of complex symlink chains."""
    files = {"file1.txt": b"content1", "dir1/": None, "dir1/file2.txt": b"content2"}
    symlinks = {"dir1/link1.txt": "../file1.txt", "dir1/link2.txt": "link1.txt"}

    with create_test_tar(files, symlinks) as tar:
        members = list(_get_stripped_tar_members(tar))
        for member in members:
            if member.islnk() or member.issym():
                normalized_link = Path(member.linkname).as_posix()
                assert not normalized_link.startswith("..")


# Version-Specific Tests
@pytest.mark.skipif(sys.version_info >= (3, 12), reason="pre-3.12 test")
def test_extract_exec_uses_safe_extractall():
    """Test pre-3.12 extraction uses safe_extractall."""
    with TemporaryDirectory() as temp_dir:
        temp_path = Path(temp_dir)
        tar_path = temp_path / "test.tar.gz"

        with tarfile.open(tar_path, "w:gz") as tar:
            info = tarfile.TarInfo("testdir/file1.txt")
            info.size = len(b"content1")
            tar.addfile(info, BytesIO(b"content1"))

        extract_dir = temp_path / "extract_safe"
        with patch("gvsbuild.utils.base_expanders._safe_extractall") as safe_mock:
            extract_exec(tar_path, extract_dir)
            assert safe_mock.called


@pytest.mark.skipif(sys.version_info < (3, 12), reason="Python 3.12+ test")
def test_extract_exec_uses_data_filter():
    """Test Python 3.12+ extraction uses data_filter."""
    with TemporaryDirectory() as temp_dir:
        temp_path = Path(temp_dir)
        tar_path = temp_path / "test.tar.gz"

        with tarfile.open(tar_path, "w:gz") as tar:
            info = tarfile.TarInfo("testdir/file1.txt")
            info.size = len(b"content1")
            tar.addfile(info, BytesIO(b"content1"))

        extract_dir = temp_path / "extract_filter"
        with patch.object(tarfile, "data_filter") as filter_mock:
            extract_exec(tar_path, extract_dir)
            assert filter_mock.called


# End-to-End Tests
def test_extract_exe(temp_workspace):
    """Test extracting an exe file."""
    extract_dir = temp_workspace / "extract_exe"
    assert extract_exec(temp_workspace / "test.exe", extract_dir)
    assert (extract_dir / "test.exe").is_file()


def test_extract_zip_strip_one(temp_workspace):
    """Test extracting a zip file with strip_one."""
    extract_dir = temp_workspace / "extract_zip_strip"
    assert extract_exec(temp_workspace / "test.zip", extract_dir, strip_one=True)
    assert (extract_dir / "file1.txt").is_file()
    assert (extract_dir / "subdir" / "file2.txt").is_file()


def test_extract_tar_strip_one(temp_workspace):
    """Test extracting a tar file with strip_one."""
    extract_dir = temp_workspace / "extract_tar_strip"
    assert extract_exec(temp_workspace / "test.tar.gz", extract_dir, strip_one=True)
    assert (extract_dir / "file1.txt").is_file()
    assert (extract_dir / "subdir" / "file2.txt").is_file()


def test_extract_with_check_file(temp_workspace):
    """Test extraction with check_file parameter."""
    extract_dir = temp_workspace / "extract_check"
    check_file = temp_workspace / "check.txt"
    check_file.touch()
    assert not extract_exec(
        temp_workspace / "test.exe", extract_dir, check_file=check_file
    )



#  Copyright (C) 2016 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.


def test_main_help(typer_app, runner):
    result = runner.invoke(typer_app, ["--help"])
    assert result.exit_code == 0
    assert "build" in result.stdout
    assert "outdated" in result.stdout
    assert "deps" in result.stdout
    assert "list" in result.stdout


def test_wrong_command(typer_app, runner):
    result = runner.invoke(typer_app, ["builds"])
    assert result.exit_code == 2
    assert "No such command" in result.stdout



#  Copyright (C) 2016 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

from gvsbuild.utils.base_builders import Meson
from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class Libpsl(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "libpsl",
            version="0.21.5",
            archive_url="https://github.com/rockdaboot/libpsl/releases/download/{version}/libpsl-{version}.tar.gz",
            hash="1dcc9ceae8b128f3c0b3f654decd0e1e891afc6ff81098f227ef260449dae208",
            dependencies=[
                "meson",
                "ninja",
                "pkgconf",
                "icu",
            ],
        )

        self.add_param("-Druntime=libicu")
        self.add_param("-Dbuiltin=true")

    def build(self):
        Meson.build(self)

        self.install(r".\LICENSE share\doc\libpsl")



#  Copyright (C) 2016 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.
import sys
from pathlib import Path

from gvsbuild.utils.base_builders import Meson
from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class PyGObject(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "pygobject",
            version="3.52.3",
            lastversion_even=True,
            repository="https://gitlab.gnome.org/GNOME/pygobject",
            archive_url="https://download.gnome.org/sources/pygobject/{major}.{minor}/pygobject-{version}.tar.gz",
            hash="00e427d291e957462a8fad659a9f9c8be776ff82a8b76bdf402f1eaeec086d82",
            dependencies=["pycairo", "gobject-introspection", "libffi"],
            patches=[
                "001-pygobject-py38-load-dll.patch",
            ],
        )

    def build(self):
        py_dir = Path(sys.executable).parent
        Meson.build(self, meson_params=f'-Dpython="{py_dir}\\python.exe"')
        gtk_dir = self.builder.gtk_dir
        add_inc = [
            str(Path(gtk_dir) / "include" / "cairo"),
            str(Path(gtk_dir) / "include" / "gobject-introspection-1.0"),
            str(Path(gtk_dir) / "include" / "glib-2.0"),
            str(Path(gtk_dir) / "lib" / "glib-2.0" / "include"),
        ]
        self.builder.mod_env("INCLUDE", ";".join(add_inc))
        if self.builder.opts.py_wheel:
            self.exec_vs(r"%(python_dir)s\python.exe -m build --wheel")
            dist_dir = Path(self.build_dir) / "dist"
            for path in dist_dir.rglob("*.whl"):
                self.exec_vs(
                    r"%(python_dir)s\python.exe -m pip install --force-reinstall "
                    + str(path)
                )
                self.install_dir("dist", "python")
        self.install(r".\COPYING share\doc\pygobject")



from gvsbuild.utils.base_builders import Meson
from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class Pcre2(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "pcre2",
            version="10.45",
            archive_url="https://github.com/PCRE2Project/pcre2/releases/download/pcre2-{version}/pcre2-{version}.tar.gz",
            hash="0e138387df7835d7403b8351e2226c1377da804e0737db0e071b48f07c9d12ee",
            dependencies=["ninja", "meson", "pkgconf"],
        )

    def build(self):
        Meson.build(self)
        self.install(r".\COPYING share\doc\pcre2")



#  Copyright (C) 2016 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.
import sys

import pytest


def test_build_help(typer_app, runner):
    result = runner.invoke(typer_app, ["build", "--help"], color=True)
    assert result.exit_code == 0
    assert "--help" in result.output


def test_wrong_project_name(typer_app, runner):
    result = runner.invoke(typer_app, ["build", "bad-name"], color=True)
    assert result.exit_code == 1
    assert "not a valid project name" in result.output


def test_no_project(typer_app, runner):
    result = runner.invoke(typer_app, ["build"])
    assert result.exit_code == 2
    assert "Missing argument" in result.output


@pytest.mark.skipif(
    not sys.platform.startswith("win"), reason="windll only available on Windows"
)
def test_platform(tmp_dir, typer_app, runner):
    assert tmp_dir.exists()
    result = runner.invoke(
        typer_app,
        [
            "build",
            "--build-dir",
            tmp_dir,
            "--platform",
            "x86",
            "hello-world",
        ],
    )
    assert result.exit_code == 0



#  Copyright (C) 2016 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

import gvsbuild.groups  # noqa: F401
import gvsbuild.projects  # noqa: F401
import gvsbuild.tools  # noqa: F401
from gvsbuild.utils.base_project import Project, ProjectType


def test_list(typer_app, runner):
    Project.add_all()
    projects = Project.list_projects()

    result = runner.invoke(typer_app, ["list"])
    assert result.exit_code == 0
    for project_type in [ProjectType.GROUP, ProjectType.PROJECT, ProjectType.TOOL]:
        for project_name in [
            project.name for project in projects if project.type == project_type
        ]:
            assert project_name in result.stdout



#  Copyright (C) 2024 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

from gvsbuild.utils.base_builders import Meson
from gvsbuild.utils.base_expanders import GitRepo
from gvsbuild.utils.base_project import Project, project_add


@project_add
class Libnice(GitRepo, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "libnice",
            version="0.1.22",
            repo_url="https://gitlab.freedesktop.org/libnice/libnice.git",
            fetch_submodules=False,
            tag="ae3eb16fd7d1237353bf64e899c612b8a63bca8a",
            dependencies=[
                "ninja",
                "meson",
            ],
        )

    def build(self):
        Meson.build(self)
        self.install(r"LICENSE share\doc\libnice")



#  Copyright (C) 2016 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

"""Base project class, used also for tools."""

import datetime
import os
import pathlib
import re
import shutil
from enum import Enum
from typing import Generic, TypeVar

from .simple_ui import log
from .utils import _rmtree_error_handler


class ProjectType(str, Enum):
    IGNORE = "ignore"
    PROJECT = "project"
    TOOL = "tool"
    GROUP = "group"


class Options:
    def __init__(self):
        self.enable_gi = False
        self.enable_fips = False
        self.ffmpeg_enable_gpl = False
        self.verbose = False
        self.debug = False
        self.platform = "x64"
        self.configuration = "release"
        self.release_configuration_is_actually_debug_optimized = False
        self.build_dir = None
        self.archives_download_dir = None
        self.export_dir = None
        self.patches_root_dir = None
        self.tools_root_dir = None
        self.vs_ver = None
        self.vs_install_path = None
        self.win_sdk_ver = None
        self.net_target_framework = None
        self.net_target_framework_version = None
        self.msys_dir = None
        self.clean = False
        self.msbuild_opts = None
        self.use_env = False
        self.deps = False
        self.check_hash = False
        self.skip = False
        self.make_zip = False
        self.zip_continue = False
        self.from_scratch = False
        self.keep_tools = False
        self.fast_build = False
        self.keep_going = False
        self.clean_built = False
        self.py_wheel = False
        self.log_size = None
        self.log_single = False
        self.cargo_opts = None
        self.ninja_opts = None
        self.extra_opts = None
        self.capture_out = False
        self.print_out = False
        self.git_expand_dir = None
        self.projects = None


P = TypeVar("P")


class Project(Generic[P]):
    def __init__(self, name, **kwargs):
        object.__init__(self)
        self.patch_dir = None
        self.build_dir = None
        self.pkg_dir = None
        self.builder = None
        self.name = name
        self.prj_dir = name
        self.dependencies = []
        self.patches = []
        self.archive_url = None
        self.archive_filename = None
        self.tarbomb = False
        self.type = None
        self.version = None
        self.repository = None
        self.lastversion_major = None
        self.lastversion_even = None
        self.internal = False
        self.mark_file = None
        self.clean = False
        self.to_add = True
        self.extra_env = {}
        self.tag = None
        self.repo_url = None
        self.extra_opts = None

        for k in kwargs:
            setattr(self, k, kwargs[k])
        self.__working_dir = None
        if len(self.name) > Project.name_len:
            Project.name_len = len(self.name)

        if not self.version:
            self.version = f"git/{self.tag}" if self.repo_url else "undefined"
        version_params = {
            "version": self.version,
            "tag": self.tag,
        }
        match = re.match(
            r"(?P<major>\d+)(\.(?P<minor>\d+))?(\.(?P<micro>\d+))?", self.version
        )
        if match:
            for param in ["major", "minor", "micro"]:
                version_params[param] = match[param] or ""

        if self.archive_url:
            self.archive_url = self.archive_url.format(**version_params)
        if self.archive_filename:
            self.archive_filename = self.archive_filename.format(**version_params)

        # register version params for use from derived classes
        self.version_params = version_params

    _projects: list[P] = []
    _names: list[str] = []
    _dict: dict[str, P] = {}
    _ver_res = None
    name_len = 0
    # List of class/type to add, now not at import time but after some options are parsed
    _reg_prj_list: list[tuple[P, ProjectType]] = []
    # build option
    opts = Options()

    @staticmethod
    def compute_dependencies(projects):
        global_deps = {p.name for p in projects}

        def _add_project_dependencies(project):
            for dep in project.dependencies:
                if dep not in global_deps:
                    global_deps.add(dep)
                    _add_project_dependencies(Project.get_project(dep))

        for project in projects:
            _add_project_dependencies(project)

        return [Project.get_project(p) for p in global_deps]

    def __str__(self):
        return self.name

    def __repr__(self):
        return repr(self.name)

    def load_defaults(self):
        # Used by the tools to load default paths/filenames
        pass

    def finalize_dep(self, builder, deps):
        """Used to manipulate the dependencies list, to add or remove projects
        For the dev-shell project is used to limit the tools to use."""
        pass

    def build(self):
        raise NotImplementedError("build")

    def post_install(self):
        pass

    def add_dependency(self, dep):
        self.dependencies.append(dep)

    def exec_cmd(self, cmd, working_dir=None, add_path=None):
        self.builder.exec_cmd(cmd, working_dir=working_dir, add_path=add_path)

    def exec_vs(self, cmd, add_path=None):
        self.builder.exec_vs(
            cmd, working_dir=self._get_working_dir(), add_path=add_path
        )

    def exec_msbuild(self, cmd, configuration=None, add_path=None):
        if not configuration:
            configuration = "%(configuration)s"
        self.exec_vs(
            "msbuild "
            + cmd
            + " /p:Configuration="
            + configuration
            + " %(msbuild_opts)s",
            add_path=add_path,
        )

    def _msbuild_make_search_replace(self, org_platform: str) -> tuple[bytes, bytes]:
        """Return the search & replace strings (converted to bytes to update
        the platform Toolset version (v140, v141, ...) to use a new compiler,
        e.g. to use vs2017 solution's files for vs2019.

        The '<PlatformToolset' at the beginning is missing to handle
        projects like libmictohttpd that has a condition in the platform
        definition
        """

        ver = self.builder.opts.vs_ver
        if ver == "15":
            dst_platform = "141"
        elif ver == "16":
            dst_platform = "142"
        elif ver == "17":
            dst_platform = "143"
        else:
            dst_platform = f"{ver}0"
        search = f">v{org_platform}</PlatformToolset>".encode()
        replace = f">v{dst_platform}</PlatformToolset>".encode()

        return search, replace

    def _msbuild_copy_dir(self, dst, src, search, replace):
        """Converts & copy a directory of a vs solution to be use with a new
        platform toolset & visual studio version.

        If dst is None the change is made in place
        """

        if dst:
            os.makedirs(dst, exist_ok=True)
            copy = True
        else:
            dst = src
            copy = False

        for cf in os.scandir(src):
            src_full = os.path.join(src, cf.name)
            dst_full = os.path.join(dst, cf.name)

            if cf.is_file():
                with open(src_full, "rb") as f:
                    content = f.read()
                new_content = content.replace(search, replace)
                if content != new_content:
                    log.info(f"File changed ({src_full})")
                    write = True
                else:
                    log.info(f"   same file ({src_full})")
                    write = copy

                if write:
                    dst_full = os.path.join(dst, cf.name)
                    with open(dst_full, "wb") as f:
                        f.write(new_content)
            elif cf.is_dir():
                self._msbuild_copy_dir(
                    dst_full if copy else None, src_full, search, replace
                )

    def exec_msbuild_gen(
        self,
        base_dir,
        sln_file,
        add_pars="",
        configuration=None,
        add_path=None,
        use_env=False,
    ):
        r"""looks for base_dir\{vs_ver}\sln_file or
        base_dir\{vs_ver_tear}\sln_file for launching the msbuild commamd.

        If it's not present in the directory the system start to look
        backward to find the first version present
        """

        def _msbuild_ok(self, dir_part):
            full = os.path.join(self.build_dir, base_dir, dir_part, sln_file)
            log.info(f"Checking for '{full}'")
            return os.path.exists(full)

        def _msbuild_copy(self, org_path, org_platform, use_ver=True):
            if use_ver:
                dst_part = f"vs{self.builder.opts.vs_ver}"
            else:
                dst_part = self.builder.vs_ver_year
            dst = os.path.join(self.build_dir, base_dir, dst_part)
            src = os.path.join(self.build_dir, base_dir, org_path)
            search, replace = self._msbuild_make_search_replace(org_platform)
            log.info(f"Vs solution copy: '{src}' -> '{dst}'")
            self._msbuild_copy_dir(dst, src, search, replace)
            return dst_part

        part = f"vs{self.builder.opts.vs_ver}"
        if not _msbuild_ok(self, part):
            part = self.builder.vs_ver_year
            if not _msbuild_ok(self, part):
                part = None

        if not part:
            look = {
                "12": [],
                "14": [
                    (
                        "vs12",
                        120,
                        True,
                    ),
                    (
                        "vs2013",
                        120,
                        False,
                    ),
                ],
                "15": [
                    (
                        "vs14",
                        140,
                        True,
                    ),
                    (
                        "vs2015",
                        140,
                        False,
                    ),
                    (
                        "vs12",
                        120,
                        True,
                    ),
                    (
                        "vs2013",
                        120,
                        False,
                    ),
                ],
                "16": [
                    (
                        "vs15",
                        141,
                        True,
                    ),
                    (
                        "vs2017",
                        141,
                        False,
                    ),
                    (
                        "vs14",
                        140,
                        True,
                    ),
                    (
                        "vs2015",
                        140,
                        False,
                    ),
                    (
                        "vs12",
                        120,
                        True,
                    ),
                    (
                        "vs2013",
                        120,
                        False,
                    ),
                ],
            }
            lst = look.get(self.builder.opts.vs_ver, [])
            for p in lst:
                if _msbuild_ok(self, p[0]):
                    # Found one, create the new directory with a copy, changing the platform identifier
                    part = _msbuild_copy(self, p[0], p[1], p[2])
                    break
            if part:
                # We log what we found because is not the default
                log.log(f"Project {self.name}, using {part} directory")

        if part:
            cmd = os.path.join(base_dir, part, sln_file)
            if add_pars:
                cmd += f" {add_pars}"
            if use_env:
                cmd += " /p:UseEnv=True"
        else:
            log.error_exit(
                f"Solution file '{sln_file}' for project '{self.name}' not found!"
            )
        self.exec_msbuild(cmd, configuration, add_path)
        return part

    def install(self, *args):
        self.builder.install(self._get_working_dir(), self.pkg_dir, *args)

    def install_dir(self, src, dest=None):
        if not dest:
            dest = os.path.basename(src)
        self.builder.install_dir(self._get_working_dir(), self.pkg_dir, src, dest)

    def install_pc_files(self, base_dir="pc-files"):
        """Install, setting dir & version, the .pc files."""
        pkgconfig_dir = os.path.join(self.pkg_dir, "lib", "pkgconfig")
        self.builder.make_dir(pkgconfig_dir)

        src_dir = os.path.join(self._get_working_dir(), base_dir)
        log.debug(f"Copy .pc files from {src_dir} to {pkgconfig_dir}")
        gtk_dir = self.builder.gtk_dir.replace("\\", "/")
        for f in os.scandir(src_dir):
            if f.is_file():
                log.debug(f" {f.name}")
                content = pathlib.Path(f.path).read_text()
                _t = content.replace("@prefix@", gtk_dir)
                content = _t
                _t = content.replace("@version@", self.version)
                content = _t

                with open(
                    os.path.join(pkgconfig_dir, f.name), "w", encoding="utf-8"
                ) as fo:
                    fo.write(content)

    def patch(self):
        for p in self.patches:
            name = os.path.basename(p)
            stamp = os.path.join(self.build_dir, f"{name}.patch-applied")
            if not os.path.exists(stamp):
                log.log(f"Applying patch {p}")
                self.builder.exec_msys(
                    ["patch", "-p1", "-i", p], working_dir=self._get_working_dir()
                )
                with open(stamp, "w", encoding="utf-8") as stampfile:
                    stampfile.write("done")
            else:
                log.debug(f"patch {p} already applied, skipping")

    def _get_working_dir(self):
        if self.__working_dir:
            return os.path.join(self.build_dir, self.__working_dir)
        else:
            return self.build_dir

    def push_location(self, path):
        self.__working_dir = path

    def pop_location(self):
        self.__working_dir = None

    def prepare_build_dir(self):
        if self.clean and os.path.exists(self.build_dir):
            shutil.rmtree(self.build_dir, onerror=_rmtree_error_handler)

        if os.path.exists(self.build_dir):
            log.debug(f"directory {self.build_dir} already exists")
            if self.update_build_dir():
                self.mark_file_remove()
                if os.path.exists(self.patch_dir):
                    log.log(f"Copying files from {self.patch_dir} to {self.build_dir}")
                    self.builder.copy_all(self.patch_dir, self.build_dir)
        else:
            self.unpack()
            if os.path.exists(self.patch_dir):
                log.log(f"Copying files from {self.patch_dir} to {self.build_dir}")
                self.builder.copy_all(self.patch_dir, self.build_dir)

    def update_build_dir(self):
        pass

    def unpack(self):
        raise NotImplementedError("unpack")

    def export(self):
        raise NotImplementedError("export")

    def get_path(self):
        # Optional for projects
        pass

    def add_extra_env(self, key, val):
        # Extra env vars for projects / tools
        self.extra_env[key] = val

    def apply_extra_env(self, base_env):
        if self.extra_env:
            for key, val in self.extra_env.items():
                if key not in base_env:
                    base_env[key] = val

    @staticmethod
    def add(proj, type=ProjectType.IGNORE):
        Project._projects.append(proj)
        Project._names.append(proj.name)
        Project._dict[proj.name] = proj
        if proj.type is None:
            proj.type = type

    @staticmethod
    def register(cls, ty):
        """Register the class to be added after some initialization."""
        Project._reg_prj_list.append(
            (
                cls,
                ty,
            )
        )

    @staticmethod
    def add_all():
        """Add all the registered class."""
        for cls, ty in Project._reg_prj_list:
            c_inst = cls()
            if c_inst.to_add:
                Project.add(c_inst, type=ty)
            else:
                del c_inst

    def ignore(self):
        """Mark the project not to build/add to the list."""
        self.to_add = False

    @staticmethod
    def get_project(name):
        try:
            return Project._dict[name]
        except KeyError:
            log.error_exit(f"Could not find project {name}")

    @staticmethod
    def list_projects():
        return list(Project._projects)

    @staticmethod
    def get_names():
        return list(Project._names)

    @staticmethod
    def get_dict():
        return dict(Project._dict)

    @staticmethod
    def get_tool_path(tool):
        if not isinstance(tool, Project):
            tool = Project._dict[tool]
        if tool.type != ProjectType.TOOL:
            return None
        t = tool.get_path()
        return t[0] or t[1] if isinstance(t, tuple) else t

    @staticmethod
    def get_tool_executable(tool):
        if not isinstance(tool, Project):
            tool = Project._dict[tool]

        return tool.get_executable() if tool.type == ProjectType.TOOL else None

    @staticmethod
    def get_tool_base_dir(tool):
        if not isinstance(tool, Project):
            tool = Project._dict[tool]

        return tool.get_base_dir() if tool.type == ProjectType.TOOL else None

    def mark_file_calc(self):
        if not self.mark_file:
            self.mark_file = os.path.join(self.build_dir, ".wingtk-built")

    def mark_file_remove(self):
        self.mark_file_calc()
        if os.path.isfile(self.mark_file):
            os.remove(self.mark_file)

    def mark_file_write(self):
        self.mark_file_calc()
        try:
            with open(self.mark_file, "w", encoding="utf-8") as fo:
                now = datetime.datetime.now().replace(microsecond=0)
                fo.write(f"{now.strftime('%Y-%m-%d %H:%M:%S')}\n")
        except FileNotFoundError as e:
            log.debug(f"Exception writing file '{self.mark_file}' ({e})")

    def mark_file_exist(self):
        rt = None
        self.mark_file_calc()
        if os.path.isfile(self.mark_file):
            try:
                with open(self.mark_file, encoding="utf-8") as fi:
                    rt = fi.readline().strip("\n")
            except OSError as e:
                print(f"Exception reading file '{self.mark_file}'")
                print(e)
        return rt

    def is_project(self):
        return self.type == ProjectType.PROJECT


def project_add(cls):
    """Class decorator to add the newly created Project class to the global
    projects/tools/groups list."""
    Project.register(cls, ProjectType.PROJECT)
    return cls


def get_project_by_type(prj_type):
    return [
        (project.name, project.version)
        for project in Project._projects
        if project.type == prj_type
    ]



#  Copyright (C) 2016 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

from gvsbuild.utils.base_builders import Meson
from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class Atk(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "atk",
            version="2.38.0",
            lastversion_even=True,
            repository="https://gitlab.gnome.org/GNOME/atk",
            archive_url="https://download.gnome.org/sources/atk/{major}.{minor}/atk-{version}.tar.xz",
            hash="ac4de2a4ef4bd5665052952fe169657e65e895c5057dffb3c2a810f6191a0c36",
            dependencies=[
                "ninja",
                "meson",
                "pkgconf",
                "glib",
            ],
        )
        if self.opts.enable_gi:
            self.add_dependency("gobject-introspection")

        self.add_param(f"-Dintrospection={self.opts.enable_gi}")
        self.add_param("-Ddocs=false")

    def build(self, **kwargs):
        Meson.build(self, make_tests=True)
        self.install(r".\COPYING share\doc\atk")



#  Copyright (C) 2016 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

from gvsbuild.utils.base_builders import Meson
from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import project_add


@project_add
class Lz4(Tarball, Meson):
    def __init__(self):
        Meson.__init__(
            self,
            "lz4",
            version="1.10.0",
            archive_url="https://github.com/lz4/lz4/archive/v{version}.tar.gz",
            archive_filename="lz4-{version}.tar.gz",
            hash="537512904744b35e232912055ccf8ec66d768639ff3abe5788d90d792ec5f48b",
            dependencies=[
                "meson",
                "ninja",
            ],
        )

        self.add_param("-Dossfuzz=false")

    def build(self):
        self.push_location(r".\build\meson")
        Meson.build(self)
        self.pop_location()

        self.install(r".\lib\LICENSE share\doc\lz4")



from gvsbuild.utils.base_builders import Meson
from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class WebrtcAudioProcessing(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "webrtc-audio-processing",
            repository="https://gitlab.freedesktop.org/pulseaudio/webrtc-audio-processing/",
            version="1.3",
            archive_url="https://freedesktop.org/software/pulseaudio/webrtc-audio-processing/webrtc-audio-processing-{version}.tar.gz",
            hash="95552fc17faa0202133707bbb3727e8c2cf64d4266fe31bfdb2298d769c1db75",
            dependencies=["meson", "ninja"],
        )

    def build(self):
        Meson.build(self)
        self.install(r".\COPYING share\doc\webrtc-audio-processing")
        self.install(
            r".\webrtc\LICENSE .\webrtc\PATENTS share\doc\webrtc-audio-processing\webrtc"
        )



#  Copyright (C) 2016 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

import os

from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add
from gvsbuild.utils.utils import convert_to_msys


@project_add
class Ffmpeg(Tarball, Project):
    def __init__(self):
        Project.__init__(
            self,
            "ffmpeg",
            version="7.1.1",
            archive_url="https://ffmpeg.org/releases/ffmpeg-{version}.tar.xz",
            hash="733984395e0dbbe5c046abda2dc49a5544e7e0e1e2366bba849222ae9e3a03b1",
            dependencies=[
                "dav1d",
                "nasm",
                "msys2",
                "pkgconf",
                "nv-codec-headers",
            ],
            patches=[],
        )
        if self.opts.ffmpeg_enable_gpl:
            self.add_dependency("x264")

    def build(self):
        configuration = (
            "debug-optimized"
            if self.opts.release_configuration_is_actually_debug_optimized
            else self.opts.configuration
        )
        msys_path = Project.get_tool_path("msys2")
        self.exec_vs(
            r"{}\bash build\build.sh {} {} {} {}".format(
                msys_path,
                convert_to_msys(self.pkg_dir),
                convert_to_msys(self.builder.gtk_dir),
                configuration,
                "enable_gpl" if self.opts.ffmpeg_enable_gpl else "disable_gpl",
            ),
            add_path=msys_path,
        )

        if configuration in ["debug-optimized", "debug"]:
            self.install(r".\libavcodec\avcodec-60.pdb bin")
            self.install(r".\libavfilter\avfilter-10.pdb bin")
            self.install(r".\libavformat\avformat-61.pdb bin")
            self.install(r".\libavutil\avutil-58.pdb bin")
            self.install(r".\libswscale\libswscale-7.pdb bin")

        self.install_pc_files()
        self.install(r".\COPYING.LGPLv2.1 " r".\COPYING.LGPLv3 " r"share\doc\ffmpeg")
        if self.opts.ffmpeg_enable_gpl:
            self.install(r".\COPYING.GPLv2 " r"share\doc\ffmpeg")

    def post_install(self):
        for lib in [
            "avcodec.lib",
            "avfilter.lib",
            "avformat.lib",
            "avutil.lib",
            "swscale.lib",
        ]:
            self.builder.exec_msys(
                ["mv", lib, "../lib/"],
                working_dir=os.path.join(self.builder.gtk_dir, "bin"),
            )


@project_add
class NvCodecHeaders(Tarball, Project):
    def __init__(self):
        Project.__init__(
            self,
            "nv-codec-headers",
            version="12.2.72.0",
            archive_url="https://github.com/FFmpeg/nv-codec-headers/releases/download/n{version}/nv-codec-headers-{version}.tar.gz",
            hash="c295a2ba8a06434d4bdc5c2208f8a825285210d71d91d572329b2c51fd0d4d03",
        )

    def build(self):
        add_path = os.path.join(self.builder.opts.msys_dir, "usr", "bin")

        self.exec_vs(r'make install PREFIX="%(gtk_dir)s"', add_path=add_path)



#  Copyright (C) 2016 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

from gvsbuild.utils.base_builders import Meson
from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class GSettingsDesktopSchemas(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "gsettings-desktop-schemas",
            version="48.0",
            repository="https://gitlab.gnome.org/GNOME/gsettings-desktop-schemas",
            archive_url="https://download.gnome.org/sources/gsettings-desktop-schemas/{major}/gsettings-desktop-schemas-{version}.tar.xz",
            hash="e68f155813bf18f865a8b2c8e9d473588b6ccadcafbb666ab788857c6c2d1bd3",
            dependencies=["meson", "ninja", "pkgconf", "glib"],
        )
        if self.opts.enable_gi:
            self.add_dependency("gobject-introspection")
            enable_gi = "true"
        else:
            enable_gi = "false"

        self.add_param(f"-Dintrospection={enable_gi}")

    def build(self):
        Meson.build(self)

        self.install(r".\COPYING share\doc\gsettings-desktop-schemas")



#  Copyright (C) 2016 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

import glob
import os

from gvsbuild.utils.base_builders import MakeGir, Meson
from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


class Project_gtk_base(Tarball, Project, MakeGir):
    def make_all_mo(self):
        if self.name == "gtk2":
            mo = "gtk20.mo"
        elif self.name == "gtk3":
            mo = "gtk30.mo"
        else:
            mo = "gtk40.mo"
        localedir = os.path.join(self.pkg_dir, "share", "locale")
        self.push_location(r".\po")
        for fp in glob.glob(os.path.join(self.build_dir, "po", "*.po")):
            f = os.path.basename(fp)
            lcmsgdir = os.path.join(localedir, f[:-3], "LC_MESSAGES")
            self.builder.make_dir(lcmsgdir)
            cmd = " ".join(["msgfmt", "-co", os.path.join(lcmsgdir, mo), f])
            self.builder.exec_cmd(cmd, working_dir=self._get_working_dir())
        self.pop_location()

        self.install(rf".\COPYING share\doc\{self.name}")


@project_add
class Gtk2(Project_gtk_base):
    def __init__(self):
        Project.__init__(
            self,
            "gtk2",
            version="2.24.33",
            lastversion_major=2,
            lastversion_even=True,
            repository="https://gitlab.gnome.org/GNOME/gtk",
            archive_url="https://download.gnome.org/sources/gtk+/{major}.{minor}/gtk+-{version}.tar.xz",
            hash="ac2ac757f5942d318a311a54b0c80b5ef295f299c2a73c632f6bfb1ff49cc6da",
            dependencies=["atk", "gdk-pixbuf", "pango"],
            patches=[
                "gtk-revert-scrolldc-commit.patch",
                "gtk-bgimg.patch",
                "gtk-accel.patch",
                # https://github.com/hexchat/hexchat/issues/1007
                "gtk-multimonitor.patch",
                # https://github.com/hexchat/hexchat/issues/2077
                "0001-GDK-W32-Remove-WS_EX_LAYERED-from-an-opaque-window.patch",
            ],
        )
        if Project.opts.enable_gi:
            self.add_dependency("gobject-introspection")

    def build(self):
        self.builder.mod_env("INCLUDE", f"{self.builder.gtk_dir}\\include\\harfbuzz")
        self.exec_msbuild_gen(r"build\win32", "gtk+.sln", add_pars="/p:UseEnv=True")

        self.make_all_mo()

    def post_install(self):
        if Project.opts.enable_gi:
            self.builder.mod_env("INCLUDE", f"{self.builder.gtk_dir}\\include\\cairo")
            self.builder.mod_env(
                "INCLUDE", f"{self.builder.gtk_dir}\\include\\harfbuzz"
            )
            self.make_single_gir("gtk2", prj_dir="gtk2")


@project_add
class Gtk3(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "gtk3",
            prj_dir="gtk3",
            version="3.24.49",
            lastversion_major=3,
            lastversion_even=True,
            repository="https://gitlab.gnome.org/GNOME/gtk",
            archive_url="https://download.gnome.org/sources/gtk/{major}.{minor}/gtk-{version}.tar.xz",
            hash="5ea52c6a28f0e5ecf2e9a3c2facbb30d040b73871fcd5f33cd1317e9018a146e",
            dependencies=["atk", "gdk-pixbuf", "pango", "libepoxy"],
            patches=[
                "gtk_update_icon_cache.patch",
            ],
        )
        if self.opts.enable_gi:
            self.add_dependency("gobject-introspection")
            enable_gi = "true"
        else:
            enable_gi = "false"

        self.add_param(f"-Dintrospection={enable_gi}")

    def build(self):
        Meson.build(self, meson_params="-Dtests=false -Ddemos=false -Dexamples=false")

        self.install(r".\COPYING share\doc\gtk3")


@project_add
class Gtk4(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "gtk4",
            prj_dir="gtk4",
            version="4.18.3",
            lastversion_major=4,
            lastversion_even=True,
            repository="https://gitlab.gnome.org/GNOME/gtk",
            archive_url="https://download.gnome.org/sources/gtk/{major}.{minor}/gtk-{version}.tar.xz",
            hash="081e1bc0b17db41a935af8d1f6f090fb1988936c42ff734d149f3d004119f8bb",
            dependencies=[
                "gdk-pixbuf",
                "pango",
                "libepoxy",
                "graphene",
                "cairo",
                "harfbuzz",
                "glib",
                "fribidi",
            ],
        )
        if self.opts.enable_gi:
            self.add_dependency("gobject-introspection")
            enable_gi = "enabled"
        else:
            enable_gi = "disabled"

        self.add_param(f"-Dintrospection={enable_gi}")
        self.add_param("-Dbuild-tests=false")
        self.add_param("-Dbuild-testsuite=false")
        self.add_param("-Dbuild-demos=false")
        self.add_param("-Dbuild-examples=false")
        self.add_param("-Dmedia-gstreamer=disabled")
        self.add_param("-Dvulkan=disabled")

    def build(self):
        Meson.build(self)
        self.install(r".\COPYING share\doc\gtk4")



#  Copyright (C) 2016 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class CyrusSasl(Tarball, Project):
    def __init__(self):
        Project.__init__(
            self,
            "cyrus-sasl",
            version="2.1.28",
            hash="9e8035c12d419209ea60584d5efa51d042c3ed44b450b9d173d5504b222df9f1",
            archive_url="https://github.com/wingtk/cyrus-sasl/releases/download/cyrus-sasl-lmdb-{version}/cyrus-sasl-{version}.tar.xz",
            dependencies=["lmdb", "openssl", "mit-kerberos"],
            patches=[
                "0001-fix-snprintf-macro.patch",
                "0001-Add-MIT-Kerberos-as-GSSAPI-provider.patch",
                "0002-Provide-a-compile-option-for-32-64-gssapi.patch",
                "0001-Fix-openssl-libs-to-point-to-the-new-openssl-1.1.1-n.patch",
            ],
        )

    def build(self):
        configuration = (
            "Debug" if self.builder.opts.configuration == "debug" else "Release"
        )
        gssapilib = "gssapi32.lib" if self.builder.x86 else "gssapi64.lib"
        self.exec_vs(
            r'nmake /nologo /f NTMakefile SASLDB="LMDB" LMDB_INCLUDE="%(gtk_dir)s\include" LMDB_LIBPATH="%(gtk_dir)s\lib" '
            + r'GSSAPI="MITKerberos" GSSAPILIB="'
            + gssapilib
            + r'" GSSAPI_INCLUDE="%(gtk_dir)s\include" GSSAPI_LIBPATH="%(gtk_dir)s\lib" '
            + r'OPENSSL_INCLUDE="%(gtk_dir)s\include" OPENSSL_LIBPATH="%(gtk_dir)s\lib" prefix="%(pkg_dir)s" CFG='
            + configuration
        )
        self.exec_vs(
            r'nmake /nologo /f NTMakefile install SASLDB="LMDB" LMDB_INCLUDE="%(gtk_dir)s\include" '
            + r'GSSAPI="MITKerberos" GSSAPILIB="'
            + gssapilib
            + r'" GSSAPI_INCLUDE="%(gtk_dir)s\include" GSSAPI_LIBPATH="%(gtk_dir)s\lib" '
            + r'LMDB_LIBPATH="%(gtk_dir)s\lib" OPENSSL_INCLUDE="%(gtk_dir)s\include" OPENSSL_LIBPATH="%(gtk_dir)s\lib" prefix="%(pkg_dir)s" CFG='
            + configuration
        )

        self.install(r".\COPYING share\doc\cyrus-sasl")
        self.install_pc_files()



#  Copyright (C) 2016 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

from gvsbuild.utils.base_builders import CmakeProject
from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class Libpng(Tarball, CmakeProject):
    def __init__(self):
        Project.__init__(
            self,
            "libpng",
            version="1.6.47",
            repository="https://github.com/pnggroup/libpng",
            archive_url="https://github.com/pnggroup/libpng/archive/v{version}.tar.gz",
            archive_filename="libpng-{version}.tar.gz",
            hash="631a4c58ea6c10c81f160c4b21fa8495b715d251698ebc2552077e8450f30454",
            dependencies=["cmake", "ninja", "zlib"],
        )

    def build(self):
        cmake_params = '-DPNG_TOOLS=OFF -DPNG_TESTS=OFF -Dld-version-script=OFF -DPNG_DEBUG_POSTFIX=""'
        CmakeProject.build(self, cmake_params=cmake_params, use_ninja=True)

        self.install_pc_files()
        self.install(r"LICENSE share\doc\libpng")



#  Copyright (C) 2016 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

from gvsbuild.utils.base_builders import Meson
from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class GdkPixbuf(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "gdk-pixbuf",
            version="2.42.12",
            repository="https://gitlab.gnome.org/GNOME/gdk-pixbuf",
            archive_url="https://download.gnome.org/sources/gdk-pixbuf/{major}.{minor}/gdk-pixbuf-{version}.tar.xz",
            hash="b9505b3445b9a7e48ced34760c3bcb73e966df3ac94c95a148cb669ab748e3c7",
            dependencies=[
                "ninja",
                "pkgconf",
                "meson",
                "libtiff-4",
                "libjpeg-turbo",
                "glib",
                "libpng",
            ],
        )
        if self.opts.enable_gi:
            self.add_dependency("gobject-introspection")
            enable_gi = "enabled"
        else:
            enable_gi = "disabled"

        self.add_param("-Dbuiltin_loaders=all")
        self.add_param(f"-Dintrospection={enable_gi}")
        self.add_param("-Dman=false")
        self.add_param("-Dtests=false")
        self.add_param("-Dgtk_doc=false")

    def build(self):
        Meson.build(self)
        self.install(r".\COPYING share\doc\gdk-pixbuf")

    def post_install(self):
        self.exec_cmd(r"%(gtk_dir)s\bin\gdk-pixbuf-query-loaders.exe --update-cache")
